import { BodyParams, PathParams, Req, Controller } from "@tsed/common";
import { Description, Get, Post, Returns, Summary } from "@tsed/schema";
import { Authenticate } from "@tsed/passport";
import { Link } from "../../entities/link/Link";
import { CreateLinkDto } from "../../entities/link/CreateLinkDTO";
import { UserService } from "../../services/UserService";
import { LinkService } from "../../services/LinkService"; // Create a new service for Link operations
import { User } from "../../entities/User";

@Controller("/links")
export class LinkController {
  constructor(
    private linkService: LinkService, // Inject LinkService
    private userService: UserService // Inject UserService
  ) {}

  @Post("/")
  @Summary("Create a new link")
  @Description("Creates a new link and associates it with a user")
  @Returns(201, Link)
  async create(@BodyParams() linkData: CreateLinkDto): Promise<Link> {
    // Delegate user creation logic to UserService
    const user = await this.userService.findOrCreate(linkData);

    // Use LinkService to handle the link creation
    return this.linkService.createLink(linkData, user);
  }

  @Get("/")
  @Authenticate("github")
  @Summary("Get all links for the authenticated user")
  @(Returns(200, Array).Of(Link))
  async getList(@Req() req: Req): Promise<Link[]> {
    const user = req.user as User;
    return this.linkService.getLinksForUser(user);
  }

  @Get("/:id")
  @Summary("Get a link by ID without text content")
  @Returns(200, Link)
  async getOne(@PathParams("id") id: string): Promise<Link | null> {
    return this.linkService.getLinkById(id);
  }

  @Get("/:id/content")
  @Authenticate("github")
  @Summary("Get the content of a link if authorized")
  @Returns(200, String)
  async getLinkContent(@PathParams("id") id: string, @Req() req: Req): Promise<string> {
    const user = req.user as User;
    return this.linkService.getLinkContentById(id, user);
  }
}
